* {
    font-family: 'Lato', sans-serif;
    border-radius: none !important;
}

a {
    text-decoration: none !important;
    cursor: pointer;
}

body {
    margin: 0;
    -webkit-font-smoothing: antialiased;
    -moz-osx-font-smoothing: grayscale;
}

code {
    font-family: source-code-pro, Menlo, Monaco, Consolas, 'Courier New',
    monospace;
}
