import React from 'react';

import './css/components.css';

import Survey from './Survey';

export default function App() {
    return (
        <div className='user-study'>
            <Survey />
        </div>
    );
}
